package com.capitalware.ufm.action.objects;

import java.io.File;
import java.util.ArrayList;

import com.ibm.mq.constants.CMQC;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.handlers.IO;
import com.capitalware.ufm.handlers.Tokens;
import com.capitalware.ufm.mq.objects.QueueObject;
import com.capitalware.ufm.objects.CWConstants;
import com.capitalware.ufm.objects.Cryptography;

/**
 * This class holds the values for the Send action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class MQSendObject extends UFMFileMQObject
{
   public String           srcFilename = "";
   public char             format = ' ';
   public String           jobXmlFile = null;
   public String           remoteJobXmlFile = null;
   public String           remoteDirectory = null;
   public String           remoteFilename = null;
   public String           archiveDirectory = CWConstants.ARCHIVE_PATH;
   public String           archiveFileName = null;
   public int              priority = CMQC.MQPRI_PRIORITY_AS_Q_DEF;
   public int              ccsid = CMQC.MQCCSI_Q_MGR;
   public boolean          persistence = true;
   public boolean          deleteFile = false;
   public boolean          archiveFile = false;
   public boolean          archiveCreateDir = false;
   public boolean          usev7prop = true;
   public Cryptography     crypto = null;
   public QueueObject      replyToQName = null;

   /**
    * The constructor
    */
   public MQSendObject()
   {
      super();
   }

   /**
    * Process tokens in Action Object's variables
    */
   public void performTokenization()
   {
      if (srcFilename != null)
         srcFilename = Tokens.replace(srcFilename, paClass.actionProperties);
      if (jobXmlFile != null)
         jobXmlFile = Tokens.replace(jobXmlFile, paClass.actionProperties);
      if (remoteDirectory != null)
         remoteDirectory = Tokens.replace(remoteDirectory, paClass.actionProperties);
      if (remoteFilename != null)
         remoteFilename = Tokens.replace(remoteFilename, paClass.actionProperties);
      if (remoteJobXmlFile != null)
         remoteJobXmlFile = Tokens.replace(remoteJobXmlFile, paClass.actionProperties);
      if (archiveDirectory != null)
         archiveDirectory = Tokens.replace(archiveDirectory, paClass.actionProperties);
      if (archiveFileName != null)
         archiveFileName = Tokens.replace(archiveFileName, paClass.actionProperties);
      
      if (replyToQName != null)
      {
         if (replyToQName.qName != null)
            replyToQName.qName = Tokens.replace(replyToQName.qName, paClass.actionProperties);

         if (replyToQName.qMgrName != null)
            replyToQName.qMgrName = Tokens.replace(replyToQName.qMgrName, paClass.actionProperties);
      }
   }

   /**
    * Check the user specified parameters.
    */
   public void checkObject()
   {
      if (remoteDirectory != null)
      {
         if ( remoteDirectory.startsWith("jobs"))
            throw new IllegalArgumentException("Security risk. Remote directory cannot be the \"jobs\" directory.");
      }

      if ( ("".equals(srcFilename)) || (srcFilename == null) )
         throw new IllegalArgumentException("Missing a required tag: File");
      else
      {
         if (srcFilename.indexOf('*') != -1)
         {
            if (srcFilename.indexOf('*') != srcFilename.lastIndexOf('*'))
               throw new IllegalArgumentException("Invalid filename wildcard: '"+srcFilename+"'. Send action only supports a single wildcard '*'. i.e. '*.txt' or 'tr*' or 'ab*.txt'");
         }
      }

      if ( deleteFile &&  archiveFile)
         throw new IllegalArgumentException("Error: User cannot specify both delete and archive together.");

      if (jobXmlFile != null)
      {
         File ljn = new File(CWConstants.JOBS_PATH + jobXmlFile);
         if (!(ljn.exists()) && onErrorFail)
               throw new IllegalArgumentException("JobFile '"+jobXmlFile+"' does not exit.");
      }

      if (crypto != null)
      {
         if ( (crypto.keySize != 128) && (crypto.keySize != 192) && (crypto.keySize != 256) )
            throw new IllegalArgumentException("Error: You must specify a keysize attribute that is 128 or 192 or 256 for the Send action.");

         if ("".equals(crypto.passPhrase))
            throw new IllegalArgumentException("Error: PassPhrase attribute cannot be blank for the Send action.");
      }

      /* set default values */
      String curDir = CWConstants.HOME_PATH;
      String filename = srcFilename;

      if (srcFilename.lastIndexOf(File.separator) != -1)
      {
         curDir = IO.getPath(srcFilename);
         filename = IO.getFileName(srcFilename);
      }

      if (remoteDirectory == null)
         remoteDirectory = curDir;

      if (remoteFilename == null)
         remoteFilename = filename;
      
      if ( (priority < -1) || (priority > 9) )
         throw new IllegalArgumentException("Error: Priority value is not valid. Valid values are -1, 0, 1, 2, 3, 4, 5, 6, 7, 8 and 9.");
      
      if (ccsid < -2)
         throw new IllegalArgumentException("Error: CCSID value is not valid.");
   }

   /**
    * Output the info related to Send Action
    * @param logger
    */
   public void printValues()
   {
      UFM.logger.info("     Format               = " + format);
      UFM.logger.info("     Priority             = " + (priority == CMQC.MQPRI_PRIORITY_AS_Q_DEF? "MQPRI_PRIORITY_AS_Q_DEF" : priority));
      UFM.logger.info("     Persistence          = " + (persistence? "Yes":"No"));
      UFM.logger.info("     CCSID                = " + (ccsid == CMQC.MQCCSI_Q_MGR? "MQCCSI_Q_MGR" : ccsid));
      UFM.logger.info("     Input Path & Filename= " + srcFilename);
      UFM.logger.info("     JobFile              = " + (jobXmlFile == null ? "" : jobXmlFile));
      UFM.logger.info("     Delete File          = " + (deleteFile?"Yes":"No"));
      UFM.logger.info("     Archive File         = " + (archiveFile?"Yes":"No"));
      UFM.logger.info("     Archive Create Dir   = " + (archiveCreateDir?"Yes":"No"));
      UFM.logger.info("     Archive Directory    = " + archiveDirectory);
      UFM.logger.info("     Archive Filename     = " + (archiveFileName == null ? "" : archiveFileName));
      UFM.logger.info("     Remote JobXmlFile    = " + (remoteJobXmlFile == null ? "" : remoteJobXmlFile));
      UFM.logger.info("     Remote Directory     = " + remoteDirectory);
      UFM.logger.info("     Remote Filename      = " + remoteFilename);
      UFM.logger.info("     ReplyToQMgrName      = " + (replyToQName==null?"":replyToQName));
      if (crypto != null)
      {
         UFM.logger.info("     KeySize              = " + crypto.keySize);
         UFM.logger.info("     PassPhrase           = " + crypto.passPhrase );
      }
   }

   /**
    * Output the info related to Send Action
    * @param logger
    */
   public void printValues(boolean showFileNames)
   {
      UFM.logger.info("     Format               = " + format);
      UFM.logger.info("     Priority             = " + (priority == CMQC.MQPRI_PRIORITY_AS_Q_DEF? "MQPRI_PRIORITY_AS_Q_DEF" : priority));
      UFM.logger.info("     Persistence          = " + (persistence? "Yes":"No"));
      UFM.logger.info("     CCSID                = " + (ccsid == CMQC.MQCCSI_Q_MGR? "MQCCSI_Q_MGR" : ccsid));
      if (showFileNames)
         UFM.logger.info("     Input Path & Filename= " + srcFilename);
      UFM.logger.info("     JobFile              = " + (jobXmlFile == null ? "" : jobXmlFile));
      UFM.logger.info("     Delete File          = " + (deleteFile?"Yes":"No"));
      UFM.logger.info("     Archive File         = " + (archiveFile?"Yes":"No"));
      UFM.logger.info("     Archive Create Dir   = " + (archiveCreateDir?"Yes":"No"));
      UFM.logger.info("     Archive Directory    = " + archiveDirectory);
      UFM.logger.info("     Archive Filename     = " + (archiveFileName == null ? "" : archiveFileName));
      UFM.logger.info("     Remote JobXmlFile    = " + (remoteJobXmlFile == null ? "" : remoteJobXmlFile));
      UFM.logger.info("     Remote Directory     = " + remoteDirectory);
      if (showFileNames)
         UFM.logger.info("     Remote Filename      = " + remoteFilename);
      UFM.logger.info("     ReplyToQMgrName      = " + (replyToQName==null?"":replyToQName));
      if (crypto != null)
      {
         UFM.logger.info("     KeySize              = " + crypto.keySize);
         UFM.logger.info("     PassPhrase           = " + crypto.passPhrase );
      }
   }
   
   /**
    * Convert the object to a string.
    * @return s
    */
   public ArrayList<String> toStringArray()
   {
      ArrayList<String> al = new ArrayList<String>();
      String file;

      if (crypto != null)
         al.add("<"+CWConstants.MQ_SEND_ELEMENT+" "+CWConstants.DELETE_ATTRIBUTE+"=\""+(deleteFile?"Y":"N")+"\" "+CWConstants.FORMAT_ATTRIBUTE+"=\""+format+"\" "+CWConstants.PRIORITY_ATTRIBUTE+"=\""+priority+"\" "+CWConstants.PERSISTENCE_ATTRIBUTE+"=\""+(persistence? "Y":"N")+"\" "+CWConstants.CCSID_ATTRIBUTE+"=\""+(ccsid == CMQC.MQCCSI_Q_MGR? "MQCCSI_Q_MGR" : ccsid)+"\" "+CWConstants.KEY_SIZE_ATTRIBUTE+"=\""+crypto.keySize+"\" "+CWConstants.PASS_PHRASE_ATTRIBUTE+"=\""+crypto.passPhrase+"\">"+LINE_SEPARATOR);
      else
         al.add("<"+CWConstants.MQ_SEND_ELEMENT+" "+CWConstants.DELETE_ATTRIBUTE+"=\""+(deleteFile?"Y":"N")+"\" "+CWConstants.FORMAT_ATTRIBUTE+"=\""+format+"\" "+CWConstants.PRIORITY_ATTRIBUTE+"=\""+priority+"\" "+CWConstants.PERSISTENCE_ATTRIBUTE+"=\""+(persistence? "Y":"N")+"\" "+CWConstants.CCSID_ATTRIBUTE+"=\""+(ccsid == CMQC.MQCCSI_Q_MGR? "MQCCSI_Q_MGR" : ccsid)+"\">"+LINE_SEPARATOR);
      
      if (srcFilename.indexOf(File.separator) != -1 )
      {
         int x = srcFilename.lastIndexOf(File.separator);
         
         String dir   = srcFilename.substring(0, x);
         String fname = srcFilename.substring(x+1);
         
         file = "  <"+CWConstants.FILE_ELEMENT+" "+CWConstants.DIR_ATTRIBUTE+"=\""+dir+"\">"+fname+"</"+CWConstants.FILE_ELEMENT+">";
      }
      else
         file = "  <"+CWConstants.FILE_ELEMENT+">"+srcFilename+"</"+CWConstants.FILE_ELEMENT+">";
      
      al.add(file+LINE_SEPARATOR);

      /*
       * MQ info
       */
      ArrayList<String> mq = createMQTag();
      for (int i=0; i < mq.size(); i++)
         al.add(mq.get(i));

      /*
       * Remote info
       */
      ArrayList<String> rt = createRemoteTag(remoteJobXmlFile, remoteDirectory, remoteFilename);
      for (int i=0; i < rt.size(); i++)
         al.add(rt.get(i));
      
      if (jobXmlFile != null)
         al.add("  <"+CWConstants.EXECUTE_ELEMENT+" "+CWConstants.XML_FILE_ATTRIBUTE+"=\""+jobXmlFile+"\" />"+LINE_SEPARATOR);
      
      if (archiveFile)
         al.add("  <"+CWConstants.ARCHIVE_ELEMENT+" "+CWConstants.CREATE_DIR_ATTRIBUTE+"=\""+(archiveCreateDir?'Y':'N')+"\" "+" "+CWConstants.TO_DIR_ATTRIBUTE+"=\""+archiveDirectory+"\" "+CWConstants.TO_FILE_ATTRIBUTE+"=\""+archiveFileName+"\" />"+LINE_SEPARATOR);
      
      al.add("    </"+CWConstants.MQ_SEND_ELEMENT+">"+LINE_SEPARATOR);
      
      return al;
   }

   /**
    * Retrieve the values in this action for the status update. toStringForStatusEventUpdate
    * @param event
    * @return s
    */
   public String toStringForStatusUpdate(String event)
   {
      String s;

      String key = "";
      if (crypto != null)
         key = CWConstants.KEY_SIZE_ATTRIBUTE+"="+crypto.keySize+", ";
      
      String rjf="";
      if (remoteJobXmlFile != null)
         rjf = "RemoteJobXmlFile="+remoteJobXmlFile+", ";

      s = CWConstants.MQ_SEND_ELEMENT+", "+labelName+", "+event+", "+createMQTagForStatusUpdate()+CWConstants.DELETE_ATTRIBUTE+"="+(deleteFile?"Y":"N")+", "+CWConstants.FORMAT_ATTRIBUTE+"="+format+", "+CWConstants.PRIORITY_ATTRIBUTE+"="+priority+", "+CWConstants.PERSISTENCE_ATTRIBUTE+"="+(persistence? "Y":"N")+", "+CWConstants.CCSID_ATTRIBUTE+"="+(ccsid == CMQC.MQCCSI_Q_MGR? "MQCCSI_Q_MGR" : ccsid)+", "+key+
          "srcFilename="+srcFilename+", "+CWConstants.ARCHIVE_ELEMENT+"="+(archiveFile?"Y":"N")+", "+CWConstants.CREATE_DIR_ATTRIBUTE+"="+(archiveCreateDir?'Y':'N')+", "+CWConstants.TO_DIR_ATTRIBUTE+"="+archiveDirectory+", "+          
          rjf+"RemoteDirectory="+remoteDirectory+", RemoteFilename="+remoteFilename+ 
          LINE_SEPARATOR;
      
      return s;
   }
}
