package com.capitalware.ufm.actions;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.jdom.JDOMException;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.action.objects.ScheduleObject;
import com.capitalware.ufm.handlers.InternalMessageQueue;
import com.capitalware.ufm.handlers.ShutdownThread;
import com.capitalware.ufm.internal.messages.UpdateStatus;
import com.capitalware.ufm.objects.PastDateException;
import com.capitalware.ufm.schedule.Holiday;
import com.capitalware.ufm.schedule.ScheduleManager;
import com.capitalware.ufm.schedule.Schedule;
import com.capitalware.ufm.schedule.ProcessScheduleEntry;
import com.capitalware.ufm.xml.HolidayXml;
import com.capitalware.ufm.xml.ScheduleXml;

/**
 * This class will handle executing an action command at a specific time.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class ScheduleAction extends UFMAction
{
   private ScheduleObject     so = null;
   private Holiday[]          holidays = null;
   private InternalMessageQueue       iq = null;
   
   /**
    * The constructor
    * @param so
    * @param iq
    */
   public ScheduleAction(ScheduleObject so, InternalMessageQueue iq)
   {
      super(so.actionName);
      this.so = so;
      this.iq = iq;
   }

   /**
    * Perform the work.
    * @return exitCode
    */
   public int doWork()
   {
      SimpleDateFormat dateTimeFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
      Schedule[] schedules = null; 
      Schedule oneSchedule = null;
      String holidayFile = null;
      int exitCode = 0;
      Thread currentThread = null;
      java.text.DecimalFormat df0 = new java.text.DecimalFormat("0000");
      String schName; 
      
      so.printValues();

      ScheduleManager sm = new ScheduleManager();

      ScheduleXml sxh = new ScheduleXml();
      sxh.setProperties(so.paClass.actionProperties);
      
      try
      {
         sxh.readXMLFile(so.xmlFile);
         schedules = sxh.getSchedules(); 
         
         holidayFile = sxh.getHolidayFile();
         if (holidayFile != null)
         {
            UFM.logger.info("     Holiday File         = " + holidayFile);

            HolidayXml hxh = new HolidayXml();
            hxh.readXMLFile(holidayFile);
            holidays = hxh.getHolidays();
         }

         if (iq != null)
            iq.addMessage(new UpdateStatus("0", dateTimeFormat.format(new Date())+", 0, "+so.toStringForStatusUpdate("Starting")));

         for (int i=0; i < schedules.length; i++)
         {
            oneSchedule = schedules[i];
            
            schName = "Schedule_"+df0.format(i+1);
            UFM.logger.info("Schedule Action: " + schName);
            oneSchedule.printValues();
            oneSchedule.checkObject();
            
            sm.addSchedule(schName, 
                           oneSchedule.minutes, 
                           oneSchedule.hours, 
                           oneSchedule.daysOfMonth, 
                           oneSchedule.months, 
                           oneSchedule.daysOfWeek, 
                           oneSchedule.year,
                           oneSchedule.xmlFile,
                           new ProcessScheduleEntry(so.actionName, so.labelName, holidays, iq));
         }
         
         /**
          * Everything is setup - Yeah!
          */
         currentThread = Thread.currentThread();
         Runtime.getRuntime().addShutdownHook(new ShutdownThread("ScheduleAction", currentThread));

         // do your work until the thread is interrupted
         try
         {
            while (!currentThread.isInterrupted())
            {
               Thread.sleep(1000);
            }
         }
         catch (InterruptedException e)
         {
            UFM.logger.info("ScheduleAction wait loop was interrupted.");
         }
      }
      catch (PastDateException e)
      {
         UFM.logger.fatal("PastDateException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 8;
      }
      catch (JDOMException e)
      {
         UFM.logger.fatal("JDOMException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 8;
      }
      catch (IOException e)
      {
         UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 8;
      }
      finally
      {
         UFM.logger.info("ScheduleAction: Beginning cleanup to exit.");

         if (sm != null)
            sm.removeAllSchedulesAndStop();

         if (currentThread != null)
         {
            UFM.logger.info("ScheduleAction: Sending notification.");
            // notify controller thread that we have finished
            synchronized (currentThread)
            {
               currentThread.notifyAll();
               UFM.logger.info("ScheduleAction: Sent notification.");
            }
         }
      }

      UFM.logger.info("ScheduleAction: All done - time to exit.");

      if (!so.onErrorFail)
         exitCode = 0; /* reset back to zero */

      return exitCode;
   }
}
