package com.capitalware.ufm.xml;

import java.io.*;
import java.util.ArrayList;
import java.util.Iterator;

import org.jdom.*;
import org.jdom.input.*;

import com.capitalware.ufm.ftp.objects.*;
import com.capitalware.ufm.handlers.StringUtilities;
import com.capitalware.ufm.objects.ConnectionObject;
import com.capitalware.ufm.xml.DTDs.ResolveLocalDTD;
import com.capitalware.ufm.objects.CWConstants;

/**
 * This class handles the reading and parsing of SFTP XML file and parse parameters
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class SFtpXml extends UFMXmlAbstract
{
   private ConnectionObject      _fco = null;
   private ArrayList<FtpCmdObject>  _cmdsAL = null;

   /**
    * The constructor
    * @param logger
    */
   public SFtpXml()
   {
      super();
   }

   /**
    * Read SFTP XML file and set parameters
    *
    * @param xmlFilename
    *           Filename of job.xml file
    * @throws IOException
    */
   public void readXMLFile(String xmlFileName) throws JDOMException, IOException
   {
      SAXBuilder builder = new SAXBuilder(true);
      // handle the ability to pull DTD from Jar if needed
      builder.setEntityResolver(new ResolveLocalDTD(CWConstants.SFTP_DTD));
      Document doc = builder.build(CWConstants.SFTP_PATH + xmlFileName);
      Element rootElement = doc.getRootElement();
      Element SFTPHostElement;
      
      _fco = new ConnectionObject();
      _fco.port = CWConstants.SFTP_DEFAULT_PORT;

      if ((SFTPHostElement = rootElement.getChild(CWConstants.HOSTNAME_ELEMENT)) != null)
         _fco.hostname = SFTPHostElement.getTextTrim();

      if ( rootElement.getChild(CWConstants.PORT_ELEMENT) != null)
      {
         String num = rootElement.getChild(CWConstants.PORT_ELEMENT).getTextTrim();
         _fco.port = StringUtilities.stringToInt(num, CWConstants.SFTP_DEFAULT_PORT);
      }

      if ( rootElement.getChild(CWConstants.USERID_ELEMENT) != null)
         _fco.userID = rootElement.getChild(CWConstants.USERID_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.PASSWORD_ELEMENT) != null)
         _fco.password = rootElement.getChild(CWConstants.PASSWORD_ELEMENT).getTextTrim();

      Element cmdsElement = rootElement.getChild(CWConstants.SFTP_CMDS_ELEMENT);
      
      _cmdsAL = extractSFtpCmds(cmdsElement);
   }

   /**
    *
    *
    * @param xmlFilename
    *           The name of xml file
    * @throws JDOMException, IOException
    */
   private ArrayList<FtpCmdObject> extractSFtpCmds(Element SFTPCmdsElement) throws JDOMException
   {
      ArrayList<FtpCmdObject> al = new ArrayList<FtpCmdObject>();

      Iterator it = SFTPCmdsElement.getChildren().iterator();

      while (it.hasNext())
      {
         Element cmdElement = (Element) it.next();
         String cmdTagName = cmdElement.getName();

         if (CWConstants.FTP_GET_ELEMENT.equals(cmdTagName))
            al.add(retrieveGet(cmdElement));
         else if  (CWConstants.FTP_PUT_ELEMENT.equals(cmdTagName))
            al.add(retrievePut( cmdElement));
         else if  (CWConstants.FTP_CD_ELEMENT.equals(cmdTagName))
            al.add(retrieveCd(cmdElement));
         else if  (CWConstants.FTP_MKDIR_ELEMENT.equals(cmdTagName))
            al.add(retrieveMkDir(cmdElement));
         else if  (CWConstants.FTP_RMDIR_ELEMENT.equals(cmdTagName))
            al.add(retrieveRmDir(cmdElement));
         else if  (CWConstants.FTP_DELETE_ELEMENT.equals(cmdTagName))
            al.add(retrieveDelete(cmdElement));
         else
            throw new JDOMException("Unknown FtpCmds tag '"+cmdTagName+"'");
      }

      return al;
   }

   /**
    * Retrieve all XML values for SFTP Get command.
    * @param cmdElement
    * @throws JDOMException
    */
   private FtpCmdObject retrieveGet(Element cmdElement)
   {
      Element fileElement = null;

      GetObject go = new GetObject();

      if ((fileElement =  cmdElement.getChild(CWConstants.FILE_ELEMENT)) != null)
      {
         String filename = fileElement.getTextTrim();
         if ( (filename != null) && (!("".equals(filename))) )         
               go.localFilename = filename;

         if (fileElement.getAttribute(CWConstants.DIR_ATTRIBUTE) != null)
            go.localDir = fileElement.getAttribute(CWConstants.DIR_ATTRIBUTE).getValue().trim();
      }

      if (cmdElement.getAttribute(CWConstants.REMOTE_DIR_ATTRIBUTE) != null)
         go.remoteDir = cmdElement.getAttribute(CWConstants.REMOTE_DIR_ATTRIBUTE).getValue().trim();

      if (cmdElement.getAttribute(CWConstants.REMOTE_FILE_ATTRIBUTE) != null)
         go.remoteFilename = cmdElement.getAttribute(CWConstants.REMOTE_FILE_ATTRIBUTE).getValue().trim();

      return go;
   }

   /**
    * Retrieve all XML values for SFTP Put command.
    * @param cmdElement
    * @throws JDOMException
    */
   private FtpCmdObject retrievePut(Element cmdElement)
   {
      Element fileElement = null;

      PutObject po = new PutObject();

//      po.localFilename = getXMLFileTags(cmdElement);
      
      if ((fileElement =  cmdElement.getChild(CWConstants.FILE_ELEMENT)) != null)
      {
         String filename = fileElement.getTextTrim();
         if ( (filename != null) && (!("".equals(filename))) )         
               po.localFilename = filename;
         
         if (fileElement.getAttribute(CWConstants.DIR_ATTRIBUTE) != null)
            po.localDir = fileElement.getAttribute(CWConstants.DIR_ATTRIBUTE).getValue().trim();
      }

      if (cmdElement.getAttribute(CWConstants.REMOTE_DIR_ATTRIBUTE) != null)
         po.remoteDir = cmdElement.getAttribute(CWConstants.REMOTE_DIR_ATTRIBUTE).getValue().trim();

      if (cmdElement.getAttribute(CWConstants.REMOTE_FILE_ATTRIBUTE) != null)
         po.remoteFilename = cmdElement.getAttribute(CWConstants.REMOTE_FILE_ATTRIBUTE).getValue().trim();

      return po;
   }

   /**
    * Retrieve all XML values for SFTP cd command.
    * @param cmdElement
    * @throws JDOMException
    */
   private FtpCmdObject retrieveCd(Element cmdElement)
   {
      CdObject mo = new CdObject();

      mo.targetDir = cmdElement.getAttribute(CWConstants.REMOTE_DIR_ATTRIBUTE).getValue().trim();

      return mo;
   }

   /**
    * Retrieve all XML values for SFTP mkdir command.
    * @param cmdElement
    * @throws JDOMException
    */
   private FtpCmdObject retrieveMkDir(Element cmdElement)
   {
      MkDirObject mdo = new MkDirObject();

      mdo.remoteDir = cmdElement.getAttribute(CWConstants.REMOTE_DIR_ATTRIBUTE).getValue().trim();

      return mdo;
   }

   /**
    * Retrieve all XML values for SFTP rmdir command.
    * @param cmdElement
    * @throws JDOMException
    */
   private FtpCmdObject retrieveRmDir(Element cmdElement)
   {
      RmDirObject rdo = new RmDirObject();

      rdo.remoteDir = cmdElement.getAttribute(CWConstants.REMOTE_DIR_ATTRIBUTE).getValue().trim();

      return rdo;
   }

   /**
    * Retrieve all XML values for SFTP rmdir command.
    * @param cmdElement
    * @throws JDOMException
    */
   private FtpCmdObject retrieveDelete(Element cmdElement)
   {
      DeleteObject dlo = new DeleteObject();

      if (cmdElement.getAttribute(CWConstants.REMOTE_DIR_ATTRIBUTE) != null)
         dlo.remoteDir = cmdElement.getAttribute(CWConstants.REMOTE_DIR_ATTRIBUTE).getValue().trim();
      
      if (cmdElement.getAttribute(CWConstants.REMOTE_FILE_ATTRIBUTE) != null)
         dlo.remoteFile = cmdElement.getAttribute(CWConstants.REMOTE_FILE_ATTRIBUTE).getValue().trim();

      return dlo;
   }

   /**
    * Retrieve the ConnectionObject.
    * @return fco
    */
   public ConnectionObject getFtpConnection()
   {
      return _fco;
   }

   /**
    * Retrieve the FtpCmdObject.
    * @return cmdsAL
    */
   public ArrayList<FtpCmdObject> getSFtpCmds()
   {
      return _cmdsAL;
   }
}
