package com.capitalware.ufm.xml;

import java.util.*;
import java.io.*;

import org.jdom.*;
import org.jdom.input.*;

import com.capitalware.ufm.handlers.Tokens;
import com.capitalware.ufm.objects.CWConstants;
import com.capitalware.ufm.schedule.Schedule;
import com.capitalware.ufm.xml.DTDs.ResolveLocalDTD;

/**
 * This class handles the reading and parsing of Schedule XML file and parse parameters
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class ScheduleXml extends UFMXmlAbstract
{
   private Schedule[]                  _schedules;
   private String                      _holidayFile = null;
   private Hashtable<String,String>    properties = null;

   /**
    * The constructor
    * @param logger
    */
   public ScheduleXml()
   {
      super();
   }

   /**
    * Read Schedule XML file and set parameters
    *
    * @param xmlFilename
    *           Filename of test_schedule.xml file
    * @throws IOException
    */
   public void readXMLFile(String xmlFilename) throws JDOMException, IOException
   {
      Schedule so = null;
      Element holidayFileElement = null;
      Element minuteElement = null;
      Element hourElement = null;
      Element dayOfMonthElement = null;
      Element monthElement = null;
      Element dayOfWeekElement = null;
      Element yearElement = null;
      
      SAXBuilder builder = new SAXBuilder(true);
      // handle the ability to pull DTD from Jar if needed
      builder.setEntityResolver(new ResolveLocalDTD(CWConstants.SCHEDULE_DTD));
      Document doc = builder.build(CWConstants.SCHEDULE_PATH + xmlFilename);
      Element rootElement = doc.getRootElement();
      
      if ( (holidayFileElement = rootElement.getChild(CWConstants.HOLIDAY_FILE_ELEMENT)) != null)
         _holidayFile = holidayFileElement.getTextTrim();
      
      // read and parse schedules
      List scheduleList = rootElement.getChildren(CWConstants.SCHEDULE_ELEMENT);
      _schedules = new Schedule[scheduleList.size()];
      int si = 0;
      Iterator it = scheduleList.iterator();
      while (it.hasNext())
      {
         Element scheduleElement = (Element) it.next();
         so = new Schedule();

         if (scheduleElement.getAttribute(CWConstants.XML_FILE_ATTRIBUTE) != null)
            so.xmlFile = scheduleElement.getAttribute(CWConstants.XML_FILE_ATTRIBUTE).getValue().trim();

         if (so.xmlFile != null)
            so.xmlFile = Tokens.replace(so.xmlFile, properties);

         if ( (minuteElement = scheduleElement.getChild(CWConstants.MINUTE_ELEMENT)) != null)
            so.minutes = so.formatStringToInts(minuteElement.getTextTrim());

         if ( (hourElement = scheduleElement.getChild(CWConstants.HOUR_ELEMENT)) != null)
            so.hours = so.formatStringToInts(hourElement.getTextTrim());

         if ( (dayOfMonthElement = scheduleElement.getChild(CWConstants.DAYOFMONTH_ELEMENT)) != null)
            so.daysOfMonth = so.formatStringToInts(dayOfMonthElement.getTextTrim());

         if ( (monthElement = scheduleElement.getChild(CWConstants.MONTH_ELEMENT)) != null)
            so.months = so.formatMonthToInts(monthElement.getTextTrim());

         if ( (dayOfWeekElement = scheduleElement.getChild(CWConstants.DAYOFWEEK_ELEMENT)) != null)
            so.daysOfWeek = so.formatDayOfWeekToInts(dayOfWeekElement.getTextTrim());
         
         if ( (yearElement = scheduleElement.getChild(CWConstants.YEAR_ELEMENT)) != null)
            so.year = so.formatStringToInt(yearElement.getTextTrim());
         
         _schedules[si++] = so;
      }
   }

   /**
    * Retrieve an array of schedules.
    * @return _schedules
    */
   public Schedule[] getSchedules()
   {
      return _schedules;
   }

   /**
    * Retrieve an holiday file.
    * @return _holidayFile
    */
   public String getHolidayFile()
   {
      return _holidayFile;
   }
   
   /**
    * Set the properties
    * @param properties
    */
   public void setProperties(Hashtable<String,String>  properties)
   {
      this.properties = properties;
   }
}
